/*
 *  PWM_control.h
 *
 *  Created on: 10.07.2024
 *      Author: Hamza
 */
 
#include "ADBMS_LL_Driver.h"
#include "TMP1075.h"
#include "soc_estimation.h"
#include "stm32f3xx_hal_def.h"
#include <eeprom.h>

// see Datasheet for these values
#define EEPROM_I2C_ADDR             0xA4    // 0xA4 for the the first 2⁸ addresses and 0xA6 for the the last 2⁸ addresses
#define EERROM_MEMORY_ADDR_SIZE     2       // it is controlled by A17 in the address Byte, see datasheet
#define EEPROM_MEMORY_SIZE          131072  // in bytes
#define EEPROM_PAGE_SIZE            32      // in bytes

static I2C_HandleTypeDef* hi2c;
uint32_t write_address, read_address;

#warning TEST THIS

void eeprom_init(I2C_HandleTypeDef* handle) {
  hi2c = handle;
  write_address = 0;
  read_address = 0;
}

void eeprom_write_status(){
  uint8_t data_length = EEPROM_PAGE_SIZE/8; //32
  uint8_t data[data_length] = {};
  
  data[0] = ((state.current_state << 4) | (current_powerground_status >> 4));
  data[1] = ((current_powerground_status << 4) | (state.error_source >> 4));        // 4 bit powerground | 4 bit error 
  data[2] = ((state.error_source << 4) | (0));                                      // 4 bit error | 4 bit state of charge
  data[3] = ((int) current_soc);                                                    // 8 bit state of charge
  data[4] = (RELAY_BAT_SIDE_VOLTAGE >> 8);                                          // 16 bit battery voltage
  data[5] = (RELAY_BAT_SIDE_VOLTAGE);                                      
  data[6] = (RELAY_ESC_SIDE_VOLTAGE >> 8);                                          // 16 bit Inverter voltage
  data[7] = (RELAY_ESC_SIDE_VOLTAGE);
  data[8] = (CURRENT_MEASUREMENT >> 8);                                             // 16 bit Inverter voltage
  data[9] = (CURRENT_MEASUREMENT);

  // data 10-35
  for (int i = 10; i < 36; i += 2) {
    data[i]  = ((int) module.auxVoltages[i]) >> 8;
    data[i+1]  = ((int) module.auxVoltages[i+1]);
  }
  for (int i = 37; i < 63; i += 2) {
    data[i] = (tmp1075_temps[i]) >> 8;
    data[i+1] = (tmp1075_temps[i+1]);
  }
  eeprom_write(data, 64);
  write_address++;
}

HAL_StatusTypeDef eeprom_read(uint8_t* data, uint16_t data_length){
  HAL_StatusTypeDef status = HAL_OK; 
  HAL_GPIO_WritePin(EEPROM___WC__GPIO_Port, EEPROM___WC__Pin, GPIO_PIN_RESET);
  for (size_t i = 0; i < data_length; i++) {
    if (read_address <= 65535){
      status = HAL_I2C_Mem_Write(
        hi2c, EEPROM_I2C_ADDR,
        read_address, EERROM_MEMORY_ADDR_SIZE,
        &data[i], 1, 10);
      
    } else if (read_address > 65535) {
        status = HAL_I2C_Mem_Write(
        hi2c, EEPROM_I2C_ADDR + 2,
        read_address, EERROM_MEMORY_ADDR_SIZE,
        &data[i], 1, 10);
    }
    read_address++; 
  }
  HAL_GPIO_WritePin(EEPROM___WC__GPIO_Port, EEPROM___WC__Pin, GPIO_PIN_SET);
  return status;
}

HAL_StatusTypeDef eeprom_write(uint8_t* data, uint16_t data_length){
  HAL_StatusTypeDef status = HAL_OK; 
  HAL_GPIO_WritePin(EEPROM___WC__GPIO_Port, EEPROM___WC__Pin, GPIO_PIN_RESET);
  for (size_t i = 0; i < data_length; i++) {
    if (write_address < 65535){
      status = HAL_I2C_Mem_Write(
        hi2c, EEPROM_I2C_ADDR,
        write_address, EERROM_MEMORY_ADDR_SIZE,
        &data[i], 1, 10);
      
    } else if (write_address > 65535) {
        status = HAL_I2C_Mem_Write(
        hi2c, EEPROM_I2C_ADDR + 2,
        write_address, EERROM_MEMORY_ADDR_SIZE,
        &data[i], 1, 10);
    }
    write_address++; 
  }
  HAL_GPIO_WritePin(EEPROM___WC__GPIO_Port, EEPROM___WC__Pin, GPIO_PIN_SET);
  return status;
}
